#!/bin/sh
# UpdateScript for Counter Gen2

# Definitions:
UBOOT_VAR_RFS_ID=rfsID
UBOOT_VAR_KERNEL_ADDR=kerneladdr
UBOOT_VAR_KERNEL_DTB_ADDR=dtbaddr
MD5SUM_FILE=md5sums.txt
KERNEL_IMAGE=uImage.zynq
RFS_IMAGE=rfs.jffs2
DTB_IMAGE=zynq-counter2.dtb
VERSION_CURRENT=`cat /version.txt`
VERSION_NEW=`cat version.txt 2>/dev/null`
UPDATE_LOGFILE=/var/log/update.log
UPDATE_HOUR_METER_LOG=/var/log/update-hour-meter.log
POSTUPDATESCRIPT_ERROROFFSET=80

# Kernel information:
KERNEL1_ADDR=0x280000
KERNEL1_MTD_NO=3
KERNEL1_DTB_ADDR=0xA00000
KERNEL1_DTB_MTD_NO=4

KERNEL2_ADDR=0xAA0000
KERNEL2_MTD_NO=5
KERNEL2_DTB_ADDR=0x1220000
KERNEL2_DTB_MTD_NO=6

# RFS information:
RFS1_ID=1f07
RFS1_ADDR=0x12C0000
RFS1_MTD_NO=7

RFS2_ID=1f08
RFS2_ADDR=0x4EC0000
RFS2_MTD_NO=8

RunPrePostScripts()
{
    script=$1
    if [ -f ${script} ] ; then
        echo "$(date): Executing script ${script} ..." >>${UPDATE_LOGFILE}
        /bin/sh ${script}
        RET=$?
        if [ "0" -ne "${RET}" ] ; then
            echo "$(date): script ${script} returned FAILURE #${RET}" >>${UPDATE_LOGFILE}
            exit `expr ${RET} + ${POSTUPDATESCRIPT_ERROROFFSET}`
        fi
        echo "$(date): script ${script} executed successfully" >>${UPDATE_LOGFILE}
    fi
}

#
# Script file name.
#
SCRIPT_NAME=$(basename $0)

# TODO required ? (replaced by firmware update compatibility check ?)
# Are we running on Counter Gen2? We check the architecture:
grep -q 'ARMv7' /proc/cpuinfo
if [ $? -ne 0 ] ; then
    logger "UpdateScript for Gen2 running on architecture other than ARMv7. Terminating."
    # exit 2 means: files not found, in this case: Gent2 MATCHING files not found:
    exit 2
fi

if [ ! -f ${MD5SUM_FILE} ] ; then
    exit 1
fi

# Test integrity by checking md5sums:
md5sum -c ${MD5SUM_FILE} >/dev/null 2>&1
if [ "0" -ne "$?" ] ; then
    # Checksum mismatch
    exit 1
fi


#
# Firmware update compatibility check. Abort if not or an error occurred.
#
COMP_CHECK_SCRIPT=FirmwareUpdateCompatibilityCheck.sh
if [ -f ${COMP_CHECK_SCRIPT} ] ; then
    /bin/sh ${COMP_CHECK_SCRIPT}
    RET=$?
    if [ "0" -ne "${RET}" ] ; then
        LOG_STR="The firmware update compatibility check failed (error code: ${RET})."
        logger -p error -t ${SCRIPT_NAME} "${LOG_STR}"
        echo "$(date): ${SCRIPT_NAME}: Error: ${LOG_STR}" >>${UPDATE_LOGFILE}
        exit 14
    fi
else
    # script not found, log and exit
    LOG_STR="The firmware update compatibility check script not found (${COMP_CHECK_SCRIPT})."
    logger -p error -t ${SCRIPT_NAME} "${LOG_STR}"
    echo "$(date): ${SCRIPT_NAME}: Error: ${LOG_STR}" >>${UPDATE_LOGFILE}
    exit 14
fi


# Commitment: We always flash all three files. Why? Because it is likely that the kernel or dtb
# will change some time. From this time on we need to flash all files anyway.
test -f ${KERNEL_IMAGE} || exit 2
test -f ${RFS_IMAGE} || exit 2
test -f ${DTB_IMAGE} || exit 2

LOGSTR="$(date): Performing a FullUpdate from version ${VERSION_CURRENT} to version ${VERSION_NEW} ..."
echo ${LOGSTR} >>${UPDATE_LOGFILE}
logger ${LOGSTR}

# Determine addresses/mtd numbers to be flashed:
RFS_ID=`fw_printenv ${UBOOT_VAR_RFS_ID} |sed -e's/.*=//' 2>/dev/null`
if [ "x${RFS_ID}" = "x${RFS1_ID}" ] ; then
    NEW_RFS_ID=${RFS2_ID}
    NEW_RFS_START_ADDRESS=${RFS2_ADDR}
    NEW_RFS_MTD_NO=${RFS2_MTD_NO}
    NEW_KERNEL_ADDR=${KERNEL2_ADDR}
    NEW_KERNEL_MTD_NO=${KERNEL2_MTD_NO}
    NEW_KERNEL_DTB_ADDR=${KERNEL2_DTB_ADDR}
    NEW_KERNEL_DTB_MTD_NO=${KERNEL2_DTB_MTD_NO}
else
    # RFS_ID is RFS2_ID or unset or set to unknown value, set it to No. 1:
    NEW_RFS_ID=${RFS1_ID}
    NEW_RFS_START_ADDRESS=${RFS1_ADDR}
    NEW_RFS_MTD_NO=${RFS1_MTD_NO}
    NEW_KERNEL_ADDR=${KERNEL1_ADDR}
    NEW_KERNEL_MTD_NO=${KERNEL1_MTD_NO}
    NEW_KERNEL_DTB_ADDR=${KERNEL1_DTB_ADDR}
    NEW_KERNEL_DTB_MTD_NO=${KERNEL1_DTB_MTD_NO}
fi

# Flash RFS image:
flash_eraseall -j -q /dev/mtd${NEW_RFS_MTD_NO} || exit 6
nandwrite -p /dev/mtd${NEW_RFS_MTD_NO} ${RFS_IMAGE} >/dev/null || exit 7

# Flash kernel image:
flash_eraseall -j -q /dev/mtd${NEW_KERNEL_MTD_NO} || exit 4
nandwrite -p /dev/mtd${NEW_KERNEL_MTD_NO} ${KERNEL_IMAGE} >/dev/null || exit 5

# Flash kernel dtb file:
flash_eraseall -j -q /dev/mtd${NEW_KERNEL_DTB_MTD_NO} || exit 4
nandwrite -p /dev/mtd${NEW_KERNEL_DTB_MTD_NO} ${DTB_IMAGE} >/dev/null || exit 5

### Possible post update scripts (scripts starting with non-capital p):
PRE_SCRIPTS=`ls p*sh 2>/dev/null`
for script in ${PRE_SCRIPTS} ; do
    RunPrePostScripts $script
done


TMPFILE=`mktemp`

# Set U-Boot environment variables for booting new images:
echo ${UBOOT_VAR_KERNEL_ADDR} ${NEW_KERNEL_ADDR}         >>${TMPFILE}
echo ${UBOOT_VAR_KERNEL_DTB_ADDR} ${NEW_KERNEL_DTB_ADDR} >>${TMPFILE}
echo ${UBOOT_VAR_RFS_ID} ${NEW_RFS_ID}                   >>${TMPFILE}
# To ensure we start from flash next time:
echo "bootcmd run bc_f"                                  >>${TMPFILE}

fw_setenv -s ${TMPFILE} >/dev/null || exit 8

# Check whether to flash BOOT.BIN:
# Start of critical section: This erase/write and the following parameter update
# must not be breaked (e.g. by power loss). Hope and pray.
if [ -f BOOT.BIN ] ; then
    flash_eraseall -j -q /dev/mtd0 || exit 4
    nandwrite -p /dev/mtd/0 BOOT.BIN >/dev/null || exit 5
fi

rm ${TMPFILE}


echo "$(date): Successfully updated kernel and rfs to version ${VERSION_NEW}" >>${UPDATE_LOGFILE}


# Patch access rights
# On some devices the access rights of the directory /var/Counter/DeviceSpecific are not set 
# correctly by the production software. 
# This will cause an error, if another user than root will connect to the device via ssh and 
# needs to read the passwd file. E.g. for creating keys with ssh-keygen or calling whoami would 
# not work in this case.
chmod 755 /var/Counter/DeviceSpecific


### Possible post update scripts (scripts starting with capital P):
POST_SCRIPTS=`ls P*sh 2>/dev/null`
for script in ${POST_SCRIPTS} ; do
    RunPrePostScripts $script
done

# reset update hour meter
# TODO: check whether this is still wanted:
/bin/rm -f ${UPDATE_HOUR_METER_LOG} >/dev/null 2>&1 
/bin/echo "0" > ${UPDATE_HOUR_METER_LOG}

# update /hwconfig/gpio.txt, if required
HW_CONFIG_DIR=/hwconfig
GPIO_FILE=gpio.txt
GPIO_PATH=${HW_CONFIG_DIR}/${GPIO_FILE}
if [ -f ${GPIO_PATH} ] ; then
	if [ "$(md5sum < ${GPIO_PATH})" = "2c8b11911b591fca9c1c6a029fc82f9e  -" ] ; then
		if [ -f ${GPIO_FILE} ] ; then
			PWD='pwd 2>/dev/null'
			mount -o remount,rw ${HW_CONFIG_DIR}
			# copy new gpio.txt
			cp -p ${GPIO_FILE} ${GPIO_PATH}
			chown root:root ${GPIO_PATH}
			# Update md5sums.txt
			cd ${HW_CONFIG_DIR}
			rm -f md5sums.txt
			md5sum *.txt > md5sums.txt
			chmod 644 md5sums.txt
			cd ${PWD}
			mount -o remount,ro ${HW_CONFIG_DIR}
			logger -p notice -t ${SCRIPT_NAME} "'${GPIO_PATH}' updated."
		else
			logger -p error -t ${SCRIPT_NAME} "No new '${GPIO_FILE}' found in update file."
		fi
	fi
else
	logger -p error -t ${SCRIPT_NAME} "Could not find '${GPIO_PATH}'."
fi

exit 0
